==> em_tasks-0.1.30-py2.py3-none-any.whl: em_tasks/tasks/bing_ads_insights_task.py <==
import os
import datetime
import csv
import json
import datetime
import pdb

import dateparser
from bingads.v13.reporting import *
from bingads.authorization import AuthorizationData
from bing_content_api.api import BingContentApi

from em_tasks import logger
from em_tasks.store.models import *


class BingAdsInsightsTask():
  def __init__(self, account_id, environment='production', version=13):
    ba_account = BingAdsAccount.get_or_none(BingAdsAccount.account_id == account_id)
    self.active = ba_account and ba_account.active
    if not self.active:
      return

    self.ba_id = ba_account.id
    self.account_id = account_id
    customer_id = ba_account.customer_id
    # self.merchant_id = ba_account.merchant_id
    client_id = ba_account.client_id
    developer_token = ba_account.developer_token
    refresh_token = ba_account.refresh_token

    api = BingContentApi(client_id, developer_token, None, refresh_token, environment)
    api.connection.authenticate()
    authorization_data = AuthorizationData(
      account_id=self.account_id,
      customer_id=customer_id,
      developer_token=developer_token,
      authentication=api.connection.auth,
    )

    self.reporting_service_manager = ReportingServiceManager(
      authorization_data=authorization_data,
      poll_interval_in_milliseconds=5000,
      environment=environment,
    )
    self.reporting_service = ServiceClient(
      service='ReportingService',
      version=version,
      authorization_data=authorization_data,
      environment=environment,
    )

    self.aggregation = 'Daily'
    self.exclude_column_headers = False
    self.exclude_report_footer = False
    self.exclude_report_header = False
    self.time = self.reporting_service.factory.create('ReportTime')
    self.time.PredefinedTime = 'Today'
    # self.time.PredefinedTime = 'Last30Days'
    self.time.ReportTimeZone = 'CentralTimeUSCanada'
    self.time.CustomDateRangeStart = None
    self.time.CustomDateRangeEnd = None
    self.return_only_complete_data = False
    self.report_file_format = 'Csv'
    self.columns = [
        'AccountName',
        'AccountNumber',
        'AccountId',
        'TimePeriod',
        'CurrencyCode',
        'Network',
        'Impressions',
        'Clicks',
        'Spend',
        'Ctr',
        'Conversions',
        'AverageCpc',
        'AverageCpm'
    ]
    self.column_cnt = len(self.columns)

    self.output_dir = os.path.join(os.path.expanduser('~'), '.em-tasks', 'bing-ads', 'reports')
    if not os.path.isdir(self.output_dir):
        os.makedirs(self.output_dir)
    self.report_filename = 'account-performance.csv'
    self.output_path = os.path.join(self.output_dir, self.report_filename)

  def run(self):
    if not self.active:
      return

    report_request = self.get_account_performance_report_request(
      account_id=self.account_id,
      aggregation=self.aggregation,
      exclude_column_headers=self.exclude_column_headers,
      exclude_report_footer=self.exclude_report_footer,
      exclude_report_header=self.exclude_report_header,
      report_file_format=self.report_file_format,
      return_only_complete_data=self.return_only_complete_data,
      time=self.time,
      columns=self.columns)
    self.download_report(report_request)

    self.extract_spend_from_report(self.output_path)

  def download_report(self, report_request):
    reporting_download_parameters = ReportingDownloadParameters(
      report_request=report_request,
      result_file_directory=self.output_dir,
      result_file_name=self.report_filename,
      overwrite_result_file = True,
      timeout_in_milliseconds=3600000
    )

    report_container = self.reporting_service_manager.download_report(reporting_download_parameters)
    if report_container is None:
      logger.error('[DownloadAccountPerformanceReportFailed] There is no report data for the submitted report request parameters.')
      return

    report_container.close()

  def get_account_performance_report_request(
    self, account_id, aggregation, exclude_column_headers, exclude_report_footer,
    exclude_report_header, report_file_format, return_only_complete_data, time, columns):
    report_request = self.reporting_service.factory.create('AccountPerformanceReportRequest')
    report_request.Aggregation = aggregation
    report_request.ExcludeColumnHeaders = exclude_column_headers
    report_request.ExcludeReportFooter = exclude_report_footer
    report_request.ExcludeReportHeader = exclude_report_header
    report_request.Format = report_file_format
    report_request.ReturnOnlyCompleteData = return_only_complete_data
    report_request.Time = time
    report_request.ReportName = "Account Performance Report"

    scope = self.reporting_service.factory.create('AccountThroughCampaignReportScope')
    scope.AccountIds = {'long': [account_id] }
    scope.Campaigns = None
    report_request.Scope = scope

    report_columns = self.reporting_service.factory.create('ArrayOfAccountPerformanceReportColumn')
    report_columns.AccountPerformanceReportColumn.append(columns)
    report_request.Columns = report_columns

    return report_request

  def extract_spend_from_report(self, report_path):
    if not report_path or not os.path.isfile(report_path):
      logger.warning('[AccountPerformanceReportNotFound] %s', report_path)
      return

    headers = None
    records = dict()
    with open(report_path, newline='') as fh:
      reader = csv.reader(fh)
      for row in reader:
        if len(row) < self.column_cnt:
          logger.debug('[NoneDataRow] %s', ','.join(row))
          continue

        if headers is None:
          headers = row
          continue

        record = dict()
        try:
          for idx, header in enumerate(headers):
            v = row[idx]
            if header == 'Ctr':
              v = round(float(v.strip('%')), 2)
            elif v.isnumeric():
              if v.find('.') == -1:
                v = int(v)
              else:
                v = round(float(v), 2)

            record[header] = v
        except:
          pass

        if 'TimePeriod' not in record:
          logger.info('[InvalidData] %s', record)
          continue

        if record.get('Network', '').find('Microsoft') == -1:
          continue

        records[record['TimePeriod']] = record

    stats = dict()
    for date_str, record in records.items():
      try:
        dt = dateparser.parse(date_str).date()
      except:
        continue

      s_date = dt.strftime('%Y-%m-%d')
      insight = {
        'impressions': record['Impressions'],
        'clicks': record['Clicks'],
        'conversions': record['Conversions'],
        'spend': record['Spend'],
        'ctr': record['Ctr'],
        'cpc': record['AverageCpc'],
        'cpm': record['AverageCpm']
      }
      ba_insight = BingAdsInsight.get_or_none((BingAdsInsight.bing_ads_account_id == self.ba_id) & (BingAdsInsight.date == s_date))
      if not ba_insight:
        ba_insight = BingAdsInsight()
        ba_insight.bing_ads_account_id = self.ba_id
        ba_insight.date = s_date

      for k, v in insight.items():
        if getattr(ba_insight, k) != v:
          insight_changed = True
          break

      if not insight_changed:
        continue

      for k, v in insight.items():
        setattr(ba_insight, k, v)
      ba_insight.save()

==> em_tasks-0.1.57-py2.py3-none-any.whl: em_tasks/tasks/bing_ads_insights_task.py <==
import os
import datetime
import csv
import json
import datetime
import pdb

import dateparser
from bingads.v13.reporting import *
from bingads.authorization import AuthorizationData
from bing_content_api.api import BingContentApi

from em_tasks import logger
from em_tasks.store.models import *


class BingAdsInsightsTask():
  def __init__(self, account_id, environment='production', version=13):
    ba_account = BingAdsAccount.get_or_none(BingAdsAccount.account_id == account_id)
    self.active = ba_account and ba_account.active
    if not self.active:
      return

    self.ba_id = ba_account.id
    self.account_id = account_id
    customer_id = ba_account.customer_id
    # self.merchant_id = ba_account.merchant_id
    client_id = ba_account.client_id
    developer_token = ba_account.developer_token
    refresh_token = ba_account.refresh_token

    api = BingContentApi(client_id, developer_token, None, refresh_token, environment)
    api.connection.authenticate()
    authorization_data = AuthorizationData(
      account_id=self.account_id,
      customer_id=customer_id,
      developer_token=developer_token,
      authentication=api.connection.auth,
    )

    self.reporting_service_manager = ReportingServiceManager(
      authorization_data=authorization_data,
      poll_interval_in_milliseconds=5000,
      environment=environment,
    )
    self.reporting_service = ServiceClient(
      service='ReportingService',
      version=version,
      authorization_data=authorization_data,
      environment=environment,
    )

    self.aggregation = 'Daily'
    self.exclude_column_headers = False
    self.exclude_report_footer = False
    self.exclude_report_header = False
    self.time = self.reporting_service.factory.create('ReportTime')
    self.time.PredefinedTime = 'Today'
    # self.time.PredefinedTime = 'Last30Days'
    self.time.ReportTimeZone = 'CentralTimeUSCanada'
    self.time.CustomDateRangeStart = None
    self.time.CustomDateRangeEnd = None
    self.return_only_complete_data = False
    self.report_file_format = 'Csv'
    self.columns = [
        'AccountName',
        'AccountNumber',
        'AccountId',
        'TimePeriod',
        'CurrencyCode',
        'Network',
        'Impressions',
        'Clicks',
        'Spend',
        'Ctr',
        'Conversions',
        'AverageCpc',
        'AverageCpm'
    ]
    self.column_cnt = len(self.columns)

    self.output_dir = os.path.join(os.path.expanduser('~'), '.em-tasks', 'bing-ads', 'reports')
    if not os.path.isdir(self.output_dir):
        os.makedirs(self.output_dir)
    self.report_filename = 'account-performance.csv'
    self.output_path = os.path.join(self.output_dir, self.report_filename)

  def run(self):
    if not self.active:
      return

    report_request = self.get_account_performance_report_request(
      account_id=self.account_id,
      aggregation=self.aggregation,
      exclude_column_headers=self.exclude_column_headers,
      exclude_report_footer=self.exclude_report_footer,
      exclude_report_header=self.exclude_report_header,
      report_file_format=self.report_file_format,
      return_only_complete_data=self.return_only_complete_data,
      time=self.time,
      columns=self.columns)
    self.download_report(report_request)

    self.extract_spend_from_report(self.output_path)

  def download_report(self, report_request):
    reporting_download_parameters = ReportingDownloadParameters(
      report_request=report_request,
      result_file_directory=self.output_dir,
      result_file_name=self.report_filename,
      overwrite_result_file = True,
      timeout_in_milliseconds=3600000
    )

    report_container = self.reporting_service_manager.download_report(reporting_download_parameters)
    if report_container is None:
      logger.error('[DownloadAccountPerformanceReportFailed] There is no report data for the submitted report request parameters.')
      return

    report_container.close()

  def get_account_performance_report_request(
    self, account_id, aggregation, exclude_column_headers, exclude_report_footer,
    exclude_report_header, report_file_format, return_only_complete_data, time, columns):
    report_request = self.reporting_service.factory.create('AccountPerformanceReportRequest')
    report_request.Aggregation = aggregation
    report_request.ExcludeColumnHeaders = exclude_column_headers
    report_request.ExcludeReportFooter = exclude_report_footer
    report_request.ExcludeReportHeader = exclude_report_header
    report_request.Format = report_file_format
    report_request.ReturnOnlyCompleteData = return_only_complete_data
    report_request.Time = time
    report_request.ReportName = "Account Performance Report"

    scope = self.reporting_service.factory.create('AccountThroughCampaignReportScope')
    scope.AccountIds = {'long': [account_id] }
    scope.Campaigns = None
    report_request.Scope = scope

    report_columns = self.reporting_service.factory.create('ArrayOfAccountPerformanceReportColumn')
    report_columns.AccountPerformanceReportColumn.append(columns)
    report_request.Columns = report_columns

    return report_request

  def extract_spend_from_report(self, report_path):
    if not report_path or not os.path.isfile(report_path):
      logger.warning('[AccountPerformanceReportNotFound] %s', report_path)
      return

    headers = None
    records = dict()
    with open(report_path, newline='') as fh:
      reader = csv.reader(fh)
      for row in reader:
        if len(row) < self.column_cnt:
          logger.debug('[NoneDataRow] %s', ','.join(row))
          continue

        if headers is None:
          headers = row
          continue

        record = dict()
        try:
          for idx, header in enumerate(headers):
            v = row[idx]
            if header == 'Ctr':
              v = round(float(v.strip('%')), 2)
            elif v.isnumeric():
              if v.find('.') == -1:
                v = int(v)
              else:
                v = round(float(v), 2)

            record[header] = v
        except:
          pass

        if 'TimePeriod' not in record:
          logger.info('[InvalidData] %s', record)
          continue

        if record.get('Network', '').find('Microsoft') == -1:
          continue

        records[record['TimePeriod']] = record

    stats = dict()
    for date_str, record in records.items():
      try:
        dt = dateparser.parse(date_str).date()
      except:
        continue

      s_date = dt.strftime('%Y-%m-%d')
      insight = {
        'impressions': record.get('Impressions', 0),
        'clicks': record.get('Clicks', 0),
        'conversions': record.get('Conversions', 0),
        'spend': record.get('Spend', 0),
        'ctr': record.get('Ctr', 0),
        'cpc': record.get('AverageCpc', 0),
        'cpm': record.get('AverageCpm', 0)
      }
      ba_insight = BingAdsInsight.get_or_none((BingAdsInsight.bing_ads_account_id == self.ba_id) & (BingAdsInsight.date == s_date))
      if not ba_insight:
        ba_insight = BingAdsInsight()
        ba_insight.bing_ads_account_id = self.ba_id
        ba_insight.date = s_date

      for k, v in insight.items():
        if getattr(ba_insight, k) != v:
          insight_changed = True
          break

      if not insight_changed:
        continue

      for k, v in insight.items():
        setattr(ba_insight, k, v)
      ba_insight.save()
